@@ -75,12 +75,24 @@ def create_user():
 def update_user():
     try:
         f = request.form
+
         attrs = {'id': f['user-id'],
                  #'username': f['user-username'],
                  'name': f['user-name'],
                  'phone': f['user-phone'],
+                 'roles': [v.replace('role-', '')
+                           for v in f if v.startswith('role-')]
                  }
 
+        if 'user-pi-select' in f:
+            attrs['pi_id'] = int(f['user-pi-select'])
+            # TODO: Validate if a user is not longer PI
+            # check that there are not other users referencing this one as pi
+            # still this will not be a very common case
+
+        for v in f:
+            print(v, '=', f[v])
+
         password = f['user-password'].strip()
         if password:
             attrs['password'] = password
@@ -124,7 +136,7 @@ def create_template():
 @api_bp.route('/get_templates', methods=['POST'])
 @flask_login.login_required
 def get_templates():
-    return filter_request(app.dm.get_applications)
+    return filter_request(app.dm.get_templates)
 
 
 @api_bp.route('/update_template', methods=['POST'])
@@ -271,7 +283,7 @@ def get_sessions():
 @api_bp.route('/create_session', methods=['POST'])
 @flask_login.login_required
 def create_session():
-    return create_item('session')
+    return handle_session(app.dm.create_session)
 
 
 @api_bp.route('/update_session', methods=['POST'])
@@ -297,7 +309,7 @@ def load_session():
 def create_session_set():
     """ Create a set file without actual session. """
     def handle(session, set_id, **attrs):
-        session.data.create_set(set_id, **attrs)
+        session.data.create_set(set_id, attrs)
         session.data.close()
         return {'session_set': {}}
 
@@ -310,7 +322,7 @@ def add_session_item():
     """ Add a new item. """
     def handle(session, set_id, **attrs):
         itemId = attrs.pop("item_id")
-        session.data.add_item(set_id, itemId, **attrs)
+        session.data.add_set_item(set_id, itemId, attrs)
         session.data.close()
         return {'item': {}}
 
@@ -323,13 +335,24 @@ def update_session_item():
     """ Update existing item. """
     def handle(session, set_id, **attrs):
         itemId = attrs.pop("item_id")
-        session.data.update_item(set_id, itemId, **attrs)
+        session.data.update_item(set_id, itemId, attrs)
         session.data.close()
         return {'item': {}}
 
     return handle_session_data(handle, mode="a")
 
 
+@api_bp.route('/get_session_data', methods=['POST'])
+@flask_login.login_required
+def get_session_data():
+    """ Return some information related to session (e.g CTF values, etc). """
+    def handle(session, set_id, **attrs):
+        result = DataContent(app).get_session_data(session)
+        session.data.close()
+        return result
+
+    return handle_session_data(handle, mode="r")
+
 # -------------------- UTILS functions ----------------------------------------
 
 def filter_request(func):
@@ -404,6 +427,12 @@ def handle_resource(resource_func):
 
 def handle_session(session_func):
     def handle(**attrs):
+        def _fix_date(date_key):
+            if date_key in attrs:
+                attrs[date_key] = datetime_from_isoformat(attrs[date_key])
+
+        _fix_date('start')
+        _fix_date('end')
         return session_func(**attrs).json()
 
     return _handle_item(handle, 'session')
