@@ -279,7 +279,7 @@ class DataManager(DbManager):
                     (e >= start and e <= end) or
                     (s <= start and e >= end))
 
-        return [b for b in self.get_bookings(condition=conditionStr)
+        return [b for b in self.get_bookings(condition=conditionStr, orderBy='start')
                 if in_range(b)]
 
     def delete_booking(self, **attrs):
@@ -415,6 +415,10 @@ class DataManager(DbManager):
 
     def __update_item(self, ModelClass, **kwargs):
         item = self.__item_by(ModelClass, id=kwargs['id'])
+        if item is None:
+            raise Exception("Not found item %s with id %s"
+                            % (ModelClass.__name__, kwargs['id']))
+
         for attr, value in kwargs.items():
             if attr != 'id':
                 setattr(item, attr, value)
@@ -449,27 +453,31 @@ class DataManager(DbManager):
         check_min_booking = kwargs.get('check_min_booking', True)
         check_max_booking = kwargs.get('check_max_booking', True)
 
-        user = self._user
-        if not user.is_manager and not r.is_active:
-            raise Exception("Selected resource is inactive now. ")
 
         if booking.start >= booking.end:
             raise Exception("The booking 'end' should be after the 'start'. ")
 
-        if not user.is_manager and booking.start.date() < self.now().date():
-            raise Exception("The booking 'start' can not be in the past. ")
+        user = self._user
 
-        if check_min_booking and r.min_booking > 0:
-            mm = dt.timedelta(minutes=int(r.min_booking * 60))
-            if booking.duration < mm:
-                raise Exception("The duration of the booking is less that "
-                                "the minimum specified for the resource. ")
+        # The following validations do not apply for managers
+        if not user.is_manager:
+            if not r.is_active:
+                raise Exception("Selected resource is inactive now. ")
 
-        if booking.type == 'booking' and check_max_booking and r.max_booking > 0:
-            mm = dt.timedelta(minutes=int(r.max_booking * 60))
-            if booking.duration > mm:
-                raise Exception("The duration of the booking is greater that "
-                                "the maximum allowed for the resource. ")
+            if booking.start.date() < self.now().date():
+                raise Exception("The booking 'start' can not be in the past. ")
+
+            if check_min_booking and r.min_booking > 0:
+                mm = dt.timedelta(minutes=int(r.min_booking * 60))
+                if booking.duration < mm:
+                    raise Exception("The duration of the booking is less that "
+                                    "the minimum specified for the resource. ")
+
+            if booking.type == 'booking' and check_max_booking and r.max_booking > 0:
+                mm = dt.timedelta(minutes=int(r.max_booking * 60))
+                if booking.duration > mm:
+                    raise Exception("The duration of the booking is greater that "
+                                    "the maximum allowed for the resource. ")
 
         overlap = self.get_bookings_range(booking.start,
                                           booking.end,
@@ -514,6 +522,12 @@ class DataManager(DbManager):
                     return None
 
                 app = find_app()
+
+                # In the case of a manager updating a booking, the manager
+                # can do the booking despise the SLOTs and Application rules
+                if app is None and user.is_manager and apps:
+                    app = apps[0]
+
                 user_can_book = any(user.can_book_slot(s) for s in overlap_slots)
 
                 if (app is None and not user.is_manager
