@@ -152,6 +152,18 @@ def create_data_models(dm):
         def requires_application(self, value):
             self.__setExtra('requires_application', bool(value))
 
+        @property
+        def daily_cost(self):
+            """ Cost of one day session using this resource.
+            """
+            return self.__getExtra('daily_cost', 0)
+
+        @daily_cost.setter
+        def daily_cost(self, value):
+            self.__setExtra('daily_cost', int(value))
+
+
+
     ApplicationUser = Table('application_user', Base.metadata,
                             Column('application_id', Integer,
                                    ForeignKey('applications.id')),
@@ -161,6 +173,7 @@ def create_data_models(dm):
     class User(UserMixin, Base):
         """Model for user accounts."""
         __tablename__ = 'users'
+        ROLES = ['user', 'admin', 'manager', 'head', 'pi']
 
         id = Column(Integer,
                     primary_key=True)
@@ -294,6 +307,11 @@ def create_data_models(dm):
         def is_active(self):
             return self.status == 'active'
 
+        @property
+        def rolesmap(self):
+            return {role: role in self.roles
+                    for role in self.ROLES}
+
         def get_pi(self):
             """ Return the PI of this user. PI are consider PI of themselves.
             """
@@ -569,6 +587,36 @@ def create_data_models(dm):
                     % (self.resource.name, self.owner.name,
                        _timestr(self.start), _timestr(self.end)))
 
+        def __getExtra(self, key, default):
+            return self.extra.get(key, default)
+
+        def __setExtra(self, key, value):
+            extra = dict(self.extra)
+            extra[key] = value
+            self.extra = extra
+
+        @property
+        def costs(self):
+            """ Return extra costs associated with this Booking
+            """
+            return  self.__getExtra('costs', [])
+
+        @costs.setter
+        def costs(self, value):
+            self.__setExtra('costs', value)
+
+        @property
+        def total_cost(self):
+            """ Return all costs associated with this Booking
+            """
+            cost = self.days * self.resource.daily_cost
+            for _, _, c in self.costs:
+                try:
+                    cost += int(c)
+                except:
+                    pass
+            return cost
+
         def json(self):
             return dm.json_from_object(self)
 
@@ -660,7 +708,7 @@ def create_data_models(dm):
 
         # User that was or is in charge of the session
         # It might be one of the facility staff or an independent user
-        operator_id = Column(Integer, ForeignKey('users.id'), nullable=False)
+        operator_id = Column(Integer, ForeignKey('users.id'), nullable=True)
         operator = relationship("User", back_populates="sessions")
 
         # General JSON dict to store extra attributes
